<?php

namespace Litepie\Deal\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepie\Deal\Http\Requests\MediumRequest;
use Litepie\Deal\Interfaces\MediumRepositoryInterface;
use Litepie\Deal\Models\Medium;

/**
 * Resource controller class for medium.
 */
class MediumResourceController extends BaseController
{

    /**
     * Initialize medium resource controller.
     *
     * @param type MediumRepositoryInterface $medium
     *
     * @return null
     */
    public function __construct(MediumRepositoryInterface $medium)
    {
        parent::__construct();
        $this->repository = $medium;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepie\Deal\Repositories\Criteria\MediumResourceCriteria::class);
    }

    /**
     * Display a list of medium.
     *
     * @return Response
     */
    public function index(MediumRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepie\Deal\Repositories\Presenter\MediumPresenter::class)
                ->$function();
        }

        $media = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('deal::medium.names'))
            ->view('deal::medium.index', true)
            ->data(compact('media', 'view'))
            ->output();
    }

    /**
     * Display medium.
     *
     * @param Request $request
     * @param Model   $medium
     *
     * @return Response
     */
    public function show(MediumRequest $request, Medium $medium)
    {

        if ($medium->exists) {
            $view = 'deal::medium.show';
        } else {
            $view = 'deal::medium.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('deal::medium.name'))
            ->data(compact('medium'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new medium.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MediumRequest $request)
    {

        $medium = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('deal::medium.name')) 
            ->view('deal::medium.create', true) 
            ->data(compact('medium'))
            ->output();
    }

    /**
     * Create new medium.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MediumRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $medium                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('deal::medium.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('deal/medium/' . $medium->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/deal/medium'))
                ->redirect();
        }

    }

    /**
     * Show medium for editing.
     *
     * @param Request $request
     * @param Model   $medium
     *
     * @return Response
     */
    public function edit(MediumRequest $request, Medium $medium)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('deal::medium.name'))
            ->view('deal::medium.edit', true)
            ->data(compact('medium'))
            ->output();
    }

    /**
     * Update the medium.
     *
     * @param Request $request
     * @param Model   $medium
     *
     * @return Response
     */
    public function update(MediumRequest $request, Medium $medium)
    {
        try {
            $attributes = $request->all();

            $medium->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('deal::medium.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('deal/medium/' . $medium->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('deal/medium/' . $medium->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the medium.
     *
     * @param Model   $medium
     *
     * @return Response
     */
    public function destroy(MediumRequest $request, Medium $medium)
    {
        try {

            $medium->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('deal::medium.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('deal/medium/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('deal/medium/' . $medium->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple medium.
     *
     * @param Model   $medium
     *
     * @return Response
     */
    public function delete(MediumRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('deal::medium.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('deal/medium'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/deal/medium'))
                ->redirect();
        }

    }

    /**
     * Restore deleted media.
     *
     * @param Model   $medium
     *
     * @return Response
     */
    public function restore(MediumRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('deal::medium.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/deal/medium'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/deal/medium/'))
                ->redirect();
        }

    }

}
